/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL1HolonomicPlatformVelocityPassThroughController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "LVL1HolonomicPlatformVelocityPassThroughController.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/interface/libraries/RTRobotUnit/LVL1Controllers/LVL1HolonomicPlatformVelocityPassThroughControllerConfig.h>

using namespace armarx;

LVL1HolonomicPlatformVelocityPassThroughController::LVL1HolonomicPlatformVelocityPassThroughController(LVL1ControllerDataProviderInterfacePtr prov, LVL1ControllerConfigPtr config)
{
    LVL1HolonomicPlatformVelocityPassThroughControllerConfigPtr cfg = LVL1HolonomicPlatformVelocityPassThroughControllerConfigPtr::dynamicCast(config);
    ARMARX_CHECK_EXPRESSION_W_HINT(cfg,
                                   "The provided config has the wrong type! The type is " << config->ice_id()
                                   << " instead of " << LVL1HolonomicPlatformVelocityPassThroughControllerConfig::ice_staticId());

    target = dynamic_cast<HolonomicPlatformVelocityTarget*>(prov->getJointTarget(cfg->platformName, ControlModes::HolonomicPlatformVelocityMode));
    ARMARX_CHECK_EXPRESSION_W_HINT(target, "The actuator " << cfg->platformName << " has no controll mode " << ControlModes::HolonomicPlatformVelocityMode);

    initialSettings.velocityX        = cfg->initialVelocityX;
    initialSettings.velocityY        = cfg->initialVelocityY;
    initialSettings.velocityRotation = cfg->initialVelocityRotation;
    reinitTripleBuffer(initialSettings);
}

void LVL1HolonomicPlatformVelocityPassThroughController::rtRun(const IceUtil::Time&, const IceUtil::Time&)
{
    target->velocityX        = rtGetControlStruct().velocityX;
    target->velocityY        = rtGetControlStruct().velocityY;
    target->velocityRotation = rtGetControlStruct().velocityRotation;
}


void LVL1HolonomicPlatformVelocityPassThroughController::setVelocites(float velocityX, float velocityY,
        float velocityRotation, const Ice::Current&)
{
    LockGuardType guard {controlDataMutex};
    getWriterControlStruct().velocityX        = velocityX;
    getWriterControlStruct().velocityY        = velocityY;
    getWriterControlStruct().velocityRotation = velocityRotation;
    writeControlStruct();
}


LVL1ControllerRegistration<LVL1HolonomicPlatformVelocityPassThroughController>
registrationLVL1HolonomicPlatformVelocityPassThroughController("LVL1HolonomicPlatformVelocityPassThroughController");
