/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL1HolonomicPlatformVelocityPassThroughController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_LVL1HolonomicPlatformVelocityPassThroughController_H
#define _ARMARX_LIB_RobotAPI_LVL1HolonomicPlatformVelocityPassThroughController_H

#include "LVL1Controller.h"
#include "../Targets/HolonomicPlatformVelocityTarget.h"
#include <RobotAPI/interface/libraries/RTRobotUnit/LVL1Controllers/LVL1HolonomicPlatformVelocityPassThroughControllerConfig.h>

namespace armarx
{
    struct LVL1HolonomicPlatformVelocityPassThroughControllerControlData
    {
        float velocityX = 0;
        float velocityY = 0;
        float velocityRotation = 0;
    };

    class LVL1HolonomicPlatformVelocityPassThroughController:
        virtual public LVL1ControllerWithTripleBuffer<LVL1HolonomicPlatformVelocityPassThroughControllerControlData>,
        virtual public LVL1HolonomicPlatformVelocityPassThroughControllerInterface
    {
    public:
        LVL1HolonomicPlatformVelocityPassThroughController(LVL1ControllerDataProviderInterfacePtr prov, LVL1ControllerConfigPtr config);

        virtual void rtRun(const IceUtil::Time&, const IceUtil::Time&) override;

        //for the platform unit
        virtual void setVelocites(float velocityX, float velocityY, float velocityRotation, const Ice::Current& = GlobalIceCurrent) override;

        //ice interface
        virtual std::string getClassName(const Ice::Current& = GlobalIceCurrent) const override
        {
            return "LVL1HolonomicPlatformVelocityPassThroughController";
        }
    protected:
        virtual void onInitComponent() override {}
        virtual void onConnectComponent() override {}

        HolonomicPlatformVelocityTarget* target;
        LVL1HolonomicPlatformVelocityPassThroughControllerControlData initialSettings;
    };
}
#endif
