/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL1KinematicUnitPositionController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_LVL1KinematicUnitPositionController_H
#define _ARMARX_LIB_RobotAPI_LVL1KinematicUnitPositionController_H

#include "LVL1Controller.h"
#include "../RobotUnit.h"
#include "../Targets/ActuatorVelocityTarget.h"

namespace armarx
{
    struct LVL1KinematicUnitPositionControllerControlSettings
    {
        //'constant' data (in some cases these variables maybe need to be changed)
        float directPControlLimit;
        float p;
        float maxDt;

        float limitPositionLo;
        float limitPositionHi;
        //variable data
        float maxVelocity;
        float acceleration;
        float deceleration;
        float positionTargetValue;
    };

    class LVL1KinematicUnitPositionController:
        virtual public LVL1ControllerWithTripleBuffer<LVL1KinematicUnitPositionControllerControlSettings>
    {
    public:
        LVL1KinematicUnitPositionController(LVL1ControllerDataProviderInterfacePtr prov, LVL1ControllerConfigPtr config);
        virtual void rtRun(const IceUtil::Time&, const IceUtil::Time& timeSinceLastIteration) override;

        //ice interface
        virtual std::string getClassName(const Ice::Current& = GlobalIceCurrent) const override
        {
            return "LVL1KinematicUnitPositionController";
        }

        //interface for kunit
        const LVL1KinematicUnitPositionControllerControlSettings& getInitialSettings()
        {
            return initialSettings;
        }
        void setTargetPosition(float pos, float maxVelocity, float acceleration, float deceleration);
        void setTargetPosition(float pos);
        void setControlSettings(LVL1KinematicUnitPositionControllerControlSettings settings);
    protected:
        virtual void onInitComponent() override {}
        virtual void onConnectComponent() override {}

        const float* currentVelocity;
        const float* currentPosition;
        ActuatorVelocityTarget* velocityTarget;
        LVL1KinematicUnitPositionControllerControlSettings initialSettings;
    };
}
#endif
