/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::LVL1KinematicUnitTorqueController
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_LVL1KinematicUnitTorqueController_H
#define _ARMARX_LIB_RobotAPI_LVL1KinematicUnitTorqueController_H

#include <atomic>

#include <boost/algorithm/clamp.hpp>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/interface/libraries/RTRobotUnit/LVL1Controllers/LVL1KinematicUnitControllerConfigs.h>

#include "LVL1Controller.h"
#include "../RobotUnit.h"

#include "../Targets/ActuatorTorqueTarget.h"

namespace armarx
{
    class LVL1KinematicUnitTorqueController:
        virtual public LVL1Controller
    {
    public:
        inline LVL1KinematicUnitTorqueController(LVL1ControllerDataProviderInterfacePtr prov, LVL1ControllerConfigPtr config);

        virtual void rtRun(const IceUtil::Time&, const IceUtil::Time&) override
        {
            torqueTarget->torque = boost::algorithm::clamp(torqueTargetValue, - maxTorque, maxTorque);
        }

        //ice interface
        inline virtual std::string getClassName(const Ice::Current& = GlobalIceCurrent) const override
        {
            return "LVL1KinematicUnitTorqueController";
        }

        //kunit
        void setMaxTorque(float mt)
        {
            maxTorque = std::abs(mt);
        }
        void setTorque(float t)
        {
            torqueTargetValue = t;
        }

        float getInitialMaxTorque() const
        {
            return initialMaxTorque;
        }

    protected:
        virtual void onInitComponent() override {}
        virtual void onConnectComponent() override {}

        std::atomic<float> maxTorque;
        std::atomic<float> torqueTargetValue;
        float initialMaxTorque;

        ActuatorTorqueTarget* torqueTarget;
    };
}
#endif
