/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::HolonomicPlatformVelocityTarget
 * @author     Raphael ( ufdrv at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_RobotAPI_HolonomicPlatformVelocityTarget_H
#define _ARMARX_LIB_RobotAPI_HolonomicPlatformVelocityTarget_H

#include "TargetBase.h"

namespace armarx
{
    /**
    * @defgroup Library-RTRobotUnit RTRobotUnit
    * @ingroup RobotAPI
    * A description of the library RTRobotUnit.
    *
    * @class HolonomicPlatformVelocityTarget
    * @ingroup Library-RTRobotUnit
    * @brief Brief description of class HolonomicPlatformVelocityTarget.
    *
    * Detailed description of class HolonomicPlatformVelocityTarget.
    */
    class HolonomicPlatformVelocityTarget: public TargetBase
    {
    public:
        float velocityX = ControllerConstants::ValueNotSetNaN;
        float velocityY = ControllerConstants::ValueNotSetNaN;
        float velocityRotation = ControllerConstants::ValueNotSetNaN;
        virtual std::string getControlMode() const override
        {
            return ControlModes::HolonomicPlatformVelocityMode;
        }
        virtual void reset() override
        {
            velocityX = ControllerConstants::ValueNotSetNaN;
            velocityY = ControllerConstants::ValueNotSetNaN;
            velocityRotation = ControllerConstants::ValueNotSetNaN;
        }
        virtual bool isValid() const override
        {
            return std::isfinite(velocityX) && std::isfinite(velocityY) && std::isfinite(velocityRotation);
        }
    };
}

#endif
